--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 22 OCT 2007																			--
-- Project name: BLAB1 firmware									 							--
--	Module name: BLAB1_TOP		  																--
--	Description : 																					--
-- 	Top level BLAB1 firmware																--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity GCC_REF is
   port ( 
			 xFCLK    		: in    std_logic;
			 xHIT    		: in    std_logic;
			 xREF				: out	  std_logic_vector(11 downto 0);
			 xGCC1			: out	  std_logic_vector(11 downto 0);
			 xGCC2			: out	  std_logic_vector(11 downto 0));		
end GCC_REF;

architecture Behavioral of GCC_REF is
	signal GCC1		: std_logic_vector(11 downto 0);	
	signal GCC2		: std_logic_vector(11 downto 0);	
	signal LOCK1	: std_logic_vector(11 downto 0);	
	signal LOCK2	: std_logic_vector(11 downto 0);	
	signal TDC1		: std_logic_vector(11 downto 0);	
	signal TDC2		: std_logic_vector(11 downto 0);	
	signal SUM		: std_logic_vector(11 downto 0);	
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------	
	component GC12_POS
   port ( 
			 PCLK    		: in    std_logic;
			 RESET	 		: in    std_logic;
			 GCODE			: out	  std_logic_vector(11 downto 0));		
   end component;
--------------------------------------------------------------------------------	
	component GC12_NEG
   port ( 
			 PCLK    		: in    std_logic;
			 RESET	 		: in    std_logic;
			 GCODE			: out	  std_logic_vector(11 downto 0));		
   end component;
--------------------------------------------------------------------------------	
	component BUF12
    Port ( 
			  I   : in   std_logic_vector (11 downto 0);
			  O	: out  std_logic_vector (11 downto 0));
   end component;
--------------------------------------------------------------------------------
	component GCCINV_12BIT
		port (
			CNT	: IN std_logic_VECTOR(11 downto 0);
			HITT	: OUT std_logic_VECTOR(11 downto 0));
		end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------	
	xGCC_coder1 : GC12_POS port map (
		PCLK  => xFCLK,
		RESET => '0',
		GCODE => GCC1);	
--------------------------------------------------------------------------------	
	xGCC_coder2 : GC12_NEG port map (
		PCLK  => xFCLK,
		RESET => '0',
		GCODE => GCC2);		
--------------------------------------------------------------------------------	
	xBUF_GCC1 : BUF12 
	port map (
		I	=> GCC1,
		O	=> xGCC1);	
--------------------------------------------------------------------------------	
	xBUF_GCC2 : BUF12 
	port map (
		I	=> GCC2,
		O	=> xGCC2);
--------------------------------------------------------------------------------	
	process(xHIT)
	begin	
		if rising_edge(xHIT) then	
			LOCK1 <= GCC1;
			LOCK2 <= GCC2;
		end if;
	end process;
--------------------------------------------------------------------------------	
	xGCCINV_1 : GCCINV_12BIT port map (
		CNT	=> LOCK1,
		HITT	=> TDC1);	
--------------------------------------------------------------------------------			
	xGCCINV_2 : GCCINV_12BIT port map (
		CNT	=> LOCK2,
		HITT	=> TDC2);	
--------------------------------------------------------------------------------			
	SUM <= TDC1 + TDC2;
--------------------------------------------------------------------------------			
	xOUTPUT_BUFFER : BUF12 port map (
		I	=> SUM,
		O	=> xREF);			
--------------------------------------------------------------------------------	
end Behavioral;