--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 10 SEPT 2007																			--
-- Project name: ICRR firmware																--
--	Module name: MESS	  							 											--
--	Description : 																					--
-- 	Master Event Synchronous Sequencer (MESS) module								--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity MESS is
   port ( 
			 xDSTRB			: in   std_logic;
			 xSEL		 		: in   std_logic;
			 xR_WR		 	: in   std_logic;
			 xDONE		 	: in   std_logic;
			 xCLR_ALL	 	: in   std_logic;
			 xEN_PED   		: in   std_logic;
			 xPED_COL_ADDR : in   std_logic_vector (4 downto 0);
			 xPED_ROW_ADDR : in   std_logic_vector (2 downto 0);
			 xADC_T			: in   std_logic_vector (15 downto 0);
			 xADC_B			: in   std_logic_vector (15 downto 0);
			 xSCALER			: in   std_logic_vector (15 downto 0);
			 xTHRES			: in   std_logic_vector (11 downto 0);
			 xROVDD			: in   std_logic_vector (11 downto 0);
			 xROW_SEL		: in   std_logic_vector (2 downto 0);
			 xCOL_SEL		: in   std_logic_vector (4 downto 0);
			 xWBIAS  		: in   std_logic_vector (11 downto 0);
			 xTERM			: in   std_logic_vector (2 downto 0);
			 xSGN	   		: in   std_logic;	
			 xHITBIT   		: in   std_logic;			 
			 xSLAVE_DATA   : out  std_logic_vector (15 downto 0);
			 xSLWR			: out  std_logic;
			 xRADDR			: out  std_logic_vector (15 downto 0));
end MESS;

architecture Behavioral of MESS is
	type STATE_TYPE is ( MEM_ADDR_SPACE,ADC_T,ADC_B,SCALER,WBIAS,
								ROVDD,THRES,FEEDBACK,GND_STATE);
	signal STATE 			: STATE_TYPE := MEM_ADDR_SPACE;
	signal RADDR			: std_logic_vector(15 downto 0) := (others=>'0');
	signal FPGA_DATA		: std_logic_vector(15 downto 0) := (others=>'0');
	signal SLWR 			: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I : in    std_logic; 
             O : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
   component BUF16
      port ( I  : in    std_logic_vector(15 downto 0); 
             O  : out   std_logic_vector(15 downto 0));
   end component;
--------------------------------------------------------------------------------
   component BUF5
      port ( I  : in    std_logic_vector(4 downto 0); 
             O  : out   std_logic_vector(4 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	process(xDSTRB,xSEL,xDONE,xCLR_ALL,xR_WR)
	begin
		if xCLR_ALL = '1' or xDONE = '1' then
			RADDR 		<= (others=>'0');
			FPGA_DATA 	<= (others=>'0');
			STATE 		<= MEM_ADDR_SPACE;
		elsif rising_edge(xDSTRB) and xSEL = '1' and xR_WR = '0' then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when MEM_ADDR_SPACE =>			
					FPGA_DATA <= xHITBIT & "000" & xROW_SEL(2 downto 0) & xCOL_SEL(4 downto 0) & x"0";
					STATE <= ADC_T;	
--------------------------------------------------------------------------------					
				when ADC_T =>			
					FPGA_DATA <= xADC_T;
					RADDR <= RADDR + 1;
					if RADDR = 4095 then --512*8 pix deep
						RADDR 		<= (others=>'0');
						STATE <= ADC_B;	
					end if;
--------------------------------------------------------------------------------					
				when ADC_B =>			
					FPGA_DATA <= xADC_B;
					RADDR <= RADDR + 1;
					if RADDR = 4095 then --512*8 pix deep
						RADDR 		<= (others=>'0');
						STATE <= SCALER;	
					end if;
--------------------------------------------------------------------------------	
				when SCALER =>			
					FPGA_DATA <= xSCALER;
					RADDR <= RADDR + 1;
					if RADDR = 2 then 
						RADDR <= (others=>'0');
						STATE <= THRES;	
					end if;
--------------------------------------------------------------------------------	
				when THRES =>			
					FPGA_DATA <= x"0" & xTHRES;
					STATE <= ROVDD;	
--------------------------------------------------------------------------------	
				when ROVDD =>			
					FPGA_DATA <= x"0" & xROVDD;
					STATE <= WBIAS;	
--------------------------------------------------------------------------------	
				when WBIAS =>			
					FPGA_DATA <= x"0" & xWBIAS;
					STATE <= FEEDBACK;	
--------------------------------------------------------------------------------	
				when FEEDBACK =>			
					FPGA_DATA <= "000" & xSGN & 
									 xPED_ROW_ADDR & xPED_COL_ADDR &  
					             xTERM(2 downto 0) & xEN_PED;
					STATE <= GND_STATE;	
--------------------------------------------------------------------------------	
				when GND_STATE =>			
					FPGA_DATA <= (others=>'0');
--------------------------------------------------------------------------------	
				when others =>	STATE<=GND_STATE;																
			end case;
		end if;
	end process;		
--------------------------------------------------------------------------------							
	xBUF_OUT : BUF16 
	port map (
		I	=> FPGA_DATA,
		O	=> xSLAVE_DATA);	
--------------------------------------------------------------------------------							
	xBUF_RADDR : BUF16 
	port map (
		I	=> RADDR,
		O	=> xRADDR);
--------------------------------------------------------------------------------							
	xBUF_SLWR : BUF 
	port map (
		I	=> SLWR,
		O	=> xSLWR);
--------------------------------------------------------------------------------							
	SLWR <= xDSTRB and xSEL and not(xR_WR); 
--------------------------------------------------------------------------------		
end Behavioral;