--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 12 JULY 2007																			--
-- Project name: BLAB1 firmware																--
--	Module name: RCO_INT		  																	--
--	Description : 																					--
-- 	Integrates the RCO from LAB3															--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity RCO_CALC is
   port ( 
			 xREFRESH  		: in    std_logic;
			 xCOMPARE  		: in    std_logic_vector (15 downto 0);			 
          xSET_VDD    	: in    std_logic_vector (11 downto 0);
			 xINT	 		  	: in    std_logic_vector (15 downto 0);
			 xCLR_ALL 		: in    std_logic;			 
			 xROVDD		  	: out   std_logic_vector (15 downto 0));
end RCO_CALC;

architecture Behavioral of RCO_CALC is
	signal xVDD				: std_logic_vector (11 downto 0);
	signal xGND				: std_logic_vector (11 downto 0);
	signal xVDD_BUF		: std_logic_vector (11 downto 0);
	signal BACK_UP_POS	: std_logic;
	signal BACK_UP_NEG	: std_logic;
	signal POS				: std_logic_vector (11 downto 0);
	signal NEG				: std_logic_vector (11 downto 0);
	signal BACK_UP			: std_logic_vector (11 downto 0);
	signal CHECK_PT		: std_logic_vector (11 downto 0);
	signal ROVDD			: std_logic_vector (15 downto 0);	
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF16
    Port ( 
			  I   : in   std_logic_vector (15 downto 0);
			  O	: out  std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------
	process(xCLR_ALL,xREFRESH)
	begin
		if xCLR_ALL = '1' then
			xVDD <= xSET_VDD;
			BACK_UP <= xSET_VDD;
		elsif rising_edge(xREFRESH) then
			if BACK_UP_POS = '1' or BACK_UP_NEG = '1' then
				xVDD <= BACK_UP;
				BACK_UP <= xVDD_BUF;
			elsif (xINT < xCOMPARE) and (xVDD > 1) then
				xVDD <= xVDD_BUF - 1; --slow down sampling
				BACK_UP <= xVDD_BUF;
			elsif (xINT > xCOMPARE) and (xVDD < 4094) then
				xVDD <= xVDD_BUF + 1; --speed up sampling
				BACK_UP <= xVDD_BUF;
			elsif (xINT = xCOMPARE) then
				xVDD <= xVDD_BUF;
				BACK_UP <= xVDD_BUF;
			end if;
		end if;
	end process;
--------------------------------------------------------------------------------
	POS <= xVDD_BUF - xVDD;
	NEG <= xVDD - xVDD_BUF;
	BACK_UP_POS <= '1' when(xVDD_BUF > xVDD and POS > 1) else '0';
	BACK_UP_NEG <= '1' when(xVDD > xVDD_BUF and NEG > 1) else '0';
--------------------------------------------------------------------------------
	process(xCLR_ALL,xREFRESH)
	begin
		if xCLR_ALL = '1' then
			xVDD_BUF <= xSET_VDD;
		elsif falling_edge(xREFRESH) then
			if BACK_UP_POS = '1' or BACK_UP_NEG = '1' then
				xVDD_BUF <= BACK_UP;
			else
				xVDD_BUF <= xVDD;
			end if;
		end if;
	end process;	
--------------------------------------------------------------------------------
	process(BACK_UP_POS,BACK_UP_NEG)--prevent error propagating to the DAC
	begin	
		if BACK_UP_POS = '1' or BACK_UP_NEG = '1' then
			CHECK_PT <= xVDD_BUF;
		else
			CHECK_PT <= xVDD;
		end if;	
	end process;
--------------------------------------------------------------------------------
   --CHECK_PT <= (others=>'1'); --Trying to set CHECK_PT to maximum to fix the sampling speed
	--ROVDD <= x"0" & CHECK_PT;
	ROVDD <= x"0fff"; -- RJN hack to fix ROVDD to set value?
--------------------------------------------------------------------------------
	xBUF_ROVDD : BUF16 
	port map (
		I	=> ROVDD,
		O	=> xROVDD);	
--------------------------------------------------------------------------------
end Behavioral;