--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 22 OCT 2007																			--
-- Project name: BLAB1 firmware									 							--
--	Module name: BLAB1_TOP		  																--
--	Description : 																					--
-- 	Top level BLAB1 firmware																--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TARGET_RAM is
port ( 
			 xT_DATA			: in    std_logic_vector (15 downto 0);
			 xB_DATA			: in    std_logic_vector (15 downto 0);
			 xW_STRB   		: in    std_logic;
			 xNRUN   		: in    std_logic;
			 xWADDR			: in    std_logic_vector (15 downto 0);
			 xSLWR			: in    std_logic;
			 xSTART    		: in    std_logic;	
			 xRADDR			: in    std_logic_vector (15 downto 0);
			 xADC_T	 		: out   std_logic_vector (15 downto 0);
			 xADC_B	 		: out   std_logic_vector (15 downto 0));
end TARGET_RAM;

architecture Behavioral of TARGET_RAM is
	signal W_EN 		: std_logic;
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
	component RAM_BLOCK
   port ( 
			 xRADDR  		: in    std_logic_vector (11 downto 0); 
          xWADDR  		: in    std_logic_vector (11 downto 0); 
          xR_CLK 			: in    std_logic; 
          xR_EN  			: in    std_logic; 
          xWRITE 			: in    std_logic_vector (15 downto 0); 
          xW_CLK 			: in    std_logic; 
          xW_EN  			: in    std_logic; 
          xREAD  			: out   std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------	
	xRAM_BLOCK_T : RAM_BLOCK 
	port map (
		xRADDR(11 downto 0)	=> xRADDR(11 downto 0),
		xWADDR(11 downto 0)	=> xWADDR(11 downto 0),
		xR_CLK 					=> xSLWR,
		xR_EN 					=> xSTART,
		xWRITE					=> xT_DATA,
		xW_CLK 					=> xW_STRB,
		xW_EN 					=> W_EN,
		xREAD					 	=> xADC_T);	
--------------------------------------------------------------------------------	
	xRAM_BLOCK_B : RAM_BLOCK 
	port map (
		xRADDR(11 downto 0)	=> xRADDR(11 downto 0),
		xWADDR(11 downto 0)	=> xWADDR(11 downto 0),
		xR_CLK 					=> xSLWR,
		xR_EN 					=> xSTART,
		xWRITE					=> xB_DATA,
		xW_CLK 					=> xW_STRB,
		xW_EN 					=> W_EN,
		xREAD					 	=> xADC_B);	
--------------------------------------------------------------------------------	
	process(xNRUN,xSTART)
	begin
		if xNRUN = '1' and xSTART = '0' then
			W_EN <= '1';
		else 
			W_EN <= '0';
		end if;
	end process;
--------------------------------------------------------------------------------	
end Behavioral;