--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 22 OCT 2007																			--
-- Project name: BLAB1 firmware			    					 							--
--	Module name: BLAB1_RAM_TIMING																--
--	Description : 																					--
-- 	BLAB1 RAM timing firmware																--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TARGET_RAMP is
   port ( 
			 xCLK    		: in  std_logic;
			 xNRUN	 		: in  std_logic;
			 xDONE	 		: in  std_logic;
			 xRESET_RAMP	: in  std_logic;
			 xSTART			: in  std_logic;
			 xCLR_ALL 		: in  std_logic;
			 xPED_COL		: in  std_logic_vector (4 downto 0);
			 xPED_ROW		: in  std_logic_vector (2 downto 0);
			 xEN_PED	 		: in  std_logic;
			 xLOCK_COL		: in  std_logic_vector(4 downto 0);
			 xLOCK_ROW		: in  std_logic_vector(2 downto 0);
			 xROW_S			: out std_logic_vector (5 downto 0);			 
			 xCOL_S			: out std_logic_vector (4 downto 0);	
			 xROW_SEL		: out std_logic_vector (2 downto 0);			 
			 xCOL_SEL		: out std_logic_vector (4 downto 0);	
			 xRAMP			: out std_logic;
			 xRAMP_DONE		: out std_logic);		
end TARGET_RAMP;

architecture Behavioral of TARGET_RAMP is
	type STATE_TYPE is ( PHASE0, PHASE1, PHASE2); 
	signal STATE			: STATE_TYPE;
	signal RAMP			   : std_logic;
	signal RAMP_DONE		: std_logic;
	signal RAMP_CNT		: std_logic_vector(15 downto 0);
	signal MAIN_CNT		: std_logic_vector(15 downto 0);
	signal ROW_SEL			: std_logic_vector(2 downto 0);
	signal COL_SEL			: std_logic_vector(4 downto 0);
	signal ROW_S			: std_logic_vector(5 downto 0);
	signal COL_S			: std_logic_vector(4 downto 0);
	signal PED_ROW			: std_logic_vector(2 downto 0);
	signal PED_COL			: std_logic_vector(4 downto 0);
	signal RAMP_ADDR		: std_logic_vector(11 downto 0);	
	signal TEMP_A			: std_logic_vector(7 downto 0);
	signal TEMP_B			: std_logic_vector(7 downto 0);
	signal TEMP_C			: std_logic_vector(7 downto 0);
	signal TEMP_D			: std_logic_vector(4 downto 0);
	signal TEMP_E			: std_logic_vector(7 downto 0);
	signal TEMP_F			: std_logic_vector(7 downto 0);
	signal LOCK_ROW		: std_logic_vector(2 downto 0);
	signal LOCK_COL		: std_logic_vector(4 downto 0);
	signal LOCK				: std_logic_vector(7 downto 0);
	signal LOCK_BACKUP	: std_logic_vector(7 downto 0);
--------------------------------------------------------------------------------
--                         look-up-table                                      --
--------------------------------------------------------------------------------
	subtype EIGHT is integer range 0 to 7;
	type ROM is array (31 downto 0) of EIGHT;
--	constant REG : ROM := (3,3,3,3,3,3,4,4,
--								  4,4,4,4,4,4,4,4,
--								  4,4,4,5,5,5,5,5,
--								  5,5,5,5,5,4,2,2);
								  
	constant REG : ROM := (4,4,4,4,4,4,4,4,
								  4,4,4,4,4,4,4,4,
								  4,4,4,5,5,5,5,5,
								  4,4,4,4,4,4,4,4);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF3
    Port ( 
			  I   : in   std_logic_vector (2 downto 0);
			  O	: out  std_logic_vector (2 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF5
    Port ( 
			  I   : in   std_logic_vector (4 downto 0);
			  O	: out  std_logic_vector (4 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component BUF6
    Port ( 
			  I   : in   std_logic_vector (5 downto 0);
			  O	: out  std_logic_vector (5 downto 0));
   end component;
--------------------------------------------------------------------------------
begin
--------------------------------------------------------------------------------	
	process(xCLK,xDONE,xCLR_ALL,xNRUN,xRESET_RAMP,xSTART)
	begin
		if xDONE = '1' or xCLR_ALL = '1' or xRESET_RAMP = '1' then
			RAMP <= '0';
			RAMP_DONE <= '0';
			RAMP_CNT <= (others=>'0');
			STATE		<= PHASE0;
		elsif rising_edge(xCLK) and xNRUN = '1' and xSTART = '0' then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when PHASE0 =>			
					RAMP_CNT <= (others=>'0');
					RAMP <= '0';
					STATE		<= PHASE1;
--------------------------------------------------------------------------------	
				when PHASE1 =>			
					RAMP_CNT <= RAMP_CNT + 1;
					RAMP <= '1';
					if RAMP_CNT = 1000 then --set RAMP length here W.R.T xCLK period = 60 MHz
						STATE		<= PHASE2;
					end if;
--------------------------------------------------------------------------------	
				when PHASE2 =>			
					RAMP <= '0';
					RAMP_DONE <= '1';
--------------------------------------------------------------------------------	
				when others =>	STATE<=PHASE0;																
			end case;
		end if;
	end process;					
--------------------------------------------------------------------------------
	process(RAMP_DONE,xCLR_ALL,xDONE)
	begin	
		if xCLR_ALL = '1' or xDONE = '1' then
			MAIN_CNT <= (others=>'0');
		elsif rising_edge(RAMP_DONE) then
			MAIN_CNT <= MAIN_CNT + 1;
		end if;
	end process;		
--------------------------------------------------------------------------------	
	RAMP_ADDR <= MAIN_CNT(11 downto 0) when(xSTART = '0') else (others=>'0');
--------------------------------------------------------------------------------	
	TEMP_A  <= "000" & xPED_COL;
	TEMP_B  <= "000" & RAMP_ADDR(4 downto 0);
	TEMP_C  <= TEMP_A + TEMP_B;
	TEMP_D  <= RAMP_ADDR(4 downto 0);
	PED_COL <= xPED_COL + TEMP_D;
--------------------------------------------------------------------------------	
	process(TEMP_C,xPED_ROW)
	begin
		if TEMP_C(5) = '1' then
			if xPED_ROW = 7 then
				PED_ROW <= "000";
			else	
				PED_ROW <= xPED_ROW + 1;
			end if;
		else
			PED_ROW <= xPED_ROW;
		end if;
	end process;
-------------------------------------------------------------------------------	
	TEMP_E  <= "000" & xLOCK_COL;
	TEMP_F  <= TEMP_E + TEMP_B;
	LOCK_COL <= xLOCK_COL + TEMP_D;
--------------------------------------------------------------------------------	
	process(TEMP_F,xLOCK_ROW)
	begin
		if TEMP_F(5) = '1' then
			if xLOCK_ROW = 7 then
				LOCK_ROW <= "000";
			else	
				LOCK_ROW <= xLOCK_ROW + 1;
			end if;
		else
			LOCK_ROW <= xLOCK_ROW;
		end if;
	end process;
-------------------------------------------------------------------------------	
	LOCK <= LOCK_ROW & LOCK_COL;
	--LOCK_BACKUP <= LOCK - REG(conv_integer(LOCK_COL(4 downto 0)));
	LOCK_BACKUP <= LOCK - 16;  -- was 16
--------------------------------------------------------------------------------	
	process(xEN_PED)
	begin	
		if xEN_PED = '1' then
			COL_SEL <= PED_COL;
			ROW_SEL <= PED_ROW;
		else
			COL_SEL <= LOCK_BACKUP(4 downto 0);
			ROW_SEL <= LOCK_BACKUP(7 downto 5);  -- handles wrap gracefully?
		end if; 
	end process; 
--------------------------------------------------------------------------------	
	ROW_S <= MAIN_CNT(7 downto 5) & ROW_SEL;
	COL_S <= COL_SEL;
--------------------------------------------------------------------------------
	xBUF_ROW_S : BUF6 
	port map (
		I	=> ROW_S, 
		O	=> xROW_S);
--------------------------------------------------------------------------------	
	xBUF_COL_S : BUF5 
	port map (
		I	=> COL_S,
		O	=> xCOL_S);	
--------------------------------------------------------------------------------
	xBUF_ROW_SEL : BUF3 
	port map (
		I	=> ROW_SEL, 
		O	=> xROW_SEL);
--------------------------------------------------------------------------------	
	xBUF_COL_SEL : BUF5 
	port map (
		I	=> COL_SEL,
		O	=> xCOL_SEL);	
--------------------------------------------------------------------------------	
	xBUF_RAMP : BUF 
	port map (
		I  => RAMP,
		O  => xRAMP);	
--------------------------------------------------------------------------------	
	xBUF_RAMP_DONE : BUF 
	port map (
		I  => RAMP_DONE,
		O  => xRAMP_DONE);	
--------------------------------------------------------------------------------	
end Behavioral;