--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.	//  Modified GSV									--
-- DATE : 22 OCT 2007 // 12-JUL-2010														--
-- Project name: BLAB1 firmware									 							--
--	Module name: TARGET_TOP		  																--
--	Description : 																					--
-- 	Top level of TARGET interface firmware												--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TARGET_TOP is
port ( 
			 xCLK    		: in  std_logic;
			 xCLK_125MHz	: in  std_logic;
			 xCLK_15MHz		: in  std_logic;
			 xFCLK    		: in  std_logic;			 
			 xTOUT			: in  std_logic_vector (15 downto 0);
			 xBOUT			: in  std_logic_vector (15 downto 0);
			 xCLR_ALL		: in  std_logic;
			 xDONE			: in  std_logic;
			 xSLWR			: in  std_logic;
			 xRADDR			: in  std_logic_vector (15 downto 0);
			 xSOFT_TRIG		: in  std_logic;
			 xEXT_TRIG 		: in  std_logic;
			 xDHIT			: in  std_logic;
			 xPED_COL		: in  std_logic_vector (4 downto 0);
			 xPED_ROW		: in  std_logic_vector (2 downto 0);
			 xEN_PED	 		: in  std_logic;
			 xSTART			: out std_logic;
			 xRAMP			: out std_logic;
			 xNRUN			: out std_logic;
			 xTRIG			: out std_logic;
			 xROW_S			: out std_logic_vector (5 downto 0);			 
			 xCOL_S			: out std_logic_vector (4 downto 0);	
			 xTTSA			: out std_logic_vector (3 downto 0);
			 xBTSA			: out std_logic_vector (3 downto 0);
			 xTSA_SEL		: out std_logic_vector (1 downto 0);
			 xROW_SEL		: out std_logic_vector (2 downto 0);			 
			 xCOL_SEL		: out std_logic_vector (4 downto 0);	
			 xMEM_LOCK		: out std_logic;				 
			 xADC_T	 		: out std_logic_vector (15 downto 0);
			 xADC_B	 		: out std_logic_vector (15 downto 0);
			 xTRACK_COL		: out std_logic_vector(4 downto 0);
			 xLOCK_ROW     : out std_logic_vector(2 downto 0);
			 xLOCK_COL		: out std_logic_vector(4 downto 0);
			 xADC_HIT		: out std_logic);
end TARGET_TOP;

architecture Behavioral of TARGET_TOP is
   attribute BOX_TYPE   : string ;
	signal START			: std_logic;
	signal RAMP			   : std_logic;
	signal xRAMP_DONE		: std_logic;
	signal xRESET_RAMP	: std_logic;
	signal WADDR			: std_logic_vector (15 downto 0);
	signal xT_DATA			: std_logic_vector (15 downto 0);
	signal xB_DATA			: std_logic_vector (15 downto 0);
	signal W_STRB			: std_logic;
	signal TRIG				: std_logic;
	signal NRUN				: std_logic;
	signal LOCK_COL		: std_logic_vector(4 downto 0);
	signal LOCK_ROW		: std_logic_vector(2 downto 0);
--------------------------------------------------------------------------------
--   								components     		   						         --
--------------------------------------------------------------------------------
   component BUF
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
   component BUFG
      port ( I  : in    std_logic;  
             O  : out   std_logic);
   end component;
--------------------------------------------------------------------------------	
	component BUF3
    Port ( 
			  I   : in   std_logic_vector (2 downto 0);
			  O	: out  std_logic_vector (2 downto 0));
   end component;
--------------------------------------------------------------------------------
	component BUF5
    Port ( 
			  I   : in   std_logic_vector (4 downto 0);
			  O	: out  std_logic_vector (4 downto 0));
   end component;
--------------------------------------------------------------------------------	
   component TARGET_TRIG
   port(  
			 xCLK		 		: in   std_logic;
			 xCLR_ALL 		: in   std_logic;
			 xDONE	 		: in   std_logic;
			 xDHIT 			: in   std_logic;
			 xSOFT_TRIG 	: in   std_logic;
			 xEXT_TRIG 		: in   std_logic;
			 xNRUN			: out  std_logic;
			 xTRIG			: out  std_logic);
   end component;
--------------------------------------------------------------------------------	
	component TARGET_TSA_CTRL
   port ( 
			 xCLK		 			: in   std_logic;
			 xCLR_ALL 			: in   std_logic;
			 xDONE	 			: in   std_logic;
			 xTRIG 				: in   std_logic;
			 xNRUN 				: in   std_logic;
			 xMEM_LOCK			: out  std_logic;
			 xLOCK_COL			: out  std_logic_vector(4 downto 0);
			 xLOCK_ROW			: out  std_logic_vector(2 downto 0);
			 xTTSA				: out  std_logic_vector (3 downto 0);
			 xBTSA				: out  std_logic_vector (3 downto 0);
			 xTSA_SEL			: out  std_logic_vector(1 downto 0);
			 xTRACK_COL			: out  std_logic_vector(4 downto 0);
			 xADC_HIT			: out  std_logic);
   end component;
--------------------------------------------------------------------------------	
	component TARGET_RAMP
   port ( 
			 xCLK    		: in  std_logic;
			 xNRUN	 		: in  std_logic;
			 xDONE	 		: in  std_logic;
			 xRESET_RAMP	: in  std_logic;
			 xSTART			: in  std_logic;
			 xCLR_ALL 		: in  std_logic;
			 xPED_COL		: in  std_logic_vector (4 downto 0);
			 xPED_ROW		: in  std_logic_vector (2 downto 0);
			 xEN_PED	 		: in  std_logic;
			 xLOCK_COL		: in  std_logic_vector(4 downto 0);
			 xLOCK_ROW		: in  std_logic_vector(2 downto 0);
			 xROW_S			: out std_logic_vector (5 downto 0);			 
			 xCOL_S			: out std_logic_vector (4 downto 0);	
			 xROW_SEL		: out std_logic_vector (2 downto 0);			 
			 xCOL_SEL		: out std_logic_vector (4 downto 0);	
			 xRAMP			: out std_logic;
			 xRAMP_DONE		: out std_logic);		
   end component;
--------------------------------------------------------------------------------	
   component TARGET_PIXx32
	port ( 
			 xFCLK			: in    std_logic;
			 xRAMP			: in    std_logic;
			 xDONE			: in    std_logic;
			 xRESET_RAMP		: in    std_logic;
			 xCLR_ALL		: in    std_logic;
			 xTOUT			: in    std_logic_vector(15 downto 0);
			 xBOUT			: in    std_logic_vector(15 downto 0);
			 xWADDR			: in    std_logic_vector(15 downto 0);
			 xT_DATA			: out	  std_logic_vector(15 downto 0);
			 xB_DATA			: out   std_logic_vector(15 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component TARGET_RAM_TIMING
   port ( 
			 xCLK    		: in    std_logic;
			 xDONE	 		: in    std_logic;
			 xNRUN	 		: in    std_logic;
			 xRAMP_DONE		: in    std_logic;
			 xCLR_ALL 		: in    std_logic;	
			 xW_STRB 		: out    std_logic;
			 xSTART	 		: out    std_logic;
			 xRESET_RAMP	: out    std_logic;
			 xADDR 		  	: out   std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
	component TARGET_RAM
	port ( 
			 xT_DATA			: in    std_logic_vector (15 downto 0);
			 xB_DATA			: in    std_logic_vector (15 downto 0);
			 xW_STRB   		: in    std_logic;
			 xNRUN   		: in    std_logic;
			 xWADDR			: in    std_logic_vector (15 downto 0);
			 xSLWR			: in    std_logic;
			 xSTART    		: in    std_logic;	
			 xRADDR			: in    std_logic_vector (15 downto 0);
			 xADC_T	 		: out   std_logic_vector (15 downto 0);
			 xADC_B	 		: out   std_logic_vector (15 downto 0));
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------	
	xTARGET_TRIG : TARGET_TRIG 
	port map (
		xCLK 				=> xCLK,
		xCLR_ALL 		=> xCLR_ALL,
		xDONE 			=> xDONE,
		xDHIT 			=> xDHIT,
		xSOFT_TRIG 		=> xSOFT_TRIG,
		xEXT_TRIG		=> xEXT_TRIG,
		xNRUN				=> NRUN,
		xTRIG 			=> TRIG);
--------------------------------------------------------------------------------	
	xTARGET_TSA_CTRL : TARGET_TSA_CTRL 
	port map (
		xCLK 				=> xCLK,
		xCLR_ALL 		=> xCLR_ALL,
		xDONE 			=> xDONE,
		xTRIG 			=> TRIG,
		xNRUN 			=> NRUN,
		xMEM_LOCK 		=> xMEM_LOCK,
		xLOCK_COL 		=> LOCK_COL,
		xLOCK_ROW		=> LOCK_ROW,
		xTTSA 			=> xTTSA,
		xBTSA				=> xBTSA,
		xTSA_SEL 		=> xTSA_SEL,
		xTRACK_COL     => xTRACK_COL,
		xADC_HIT       => xADC_HIT);
--------------------------------------------------------------------------------	
	xTARGET_RAMP : TARGET_RAMP 
	port map (
		xCLK 				=> xCLK_15MHz,--xCLK
		xNRUN 			=> NRUN,
		xDONE 			=> xDONE,
		xRESET_RAMP 	=> xRESET_RAMP,
		xSTART 			=> START,
		xCLR_ALL 		=> xCLR_ALL,
		xPED_COL			=> xPED_COL,
		xPED_ROW			=> xPED_ROW,
		xEN_PED			=> xEN_PED,
		xROW_SEL 		=> xROW_SEL,
		xCOL_SEL 		=> xCOL_SEL,
		xLOCK_COL 		=> LOCK_COL,
		xLOCK_ROW		=> LOCK_ROW,
		xROW_S 			=> xROW_S,
		xCOL_S 			=> xCOL_S,
		xRAMP 			=> RAMP,
		xRAMP_DONE 		=> xRAMP_DONE);
--------------------------------------------------------------------------------
	xTARGET_PIXx32 : TARGET_PIXx32
	port map (
		xFCLK 	=> xCLK_125MHz,--xFCLK  doesn't meet timing constraint == GSV
		xRAMP 	=> RAMP,
		xDONE 	=> xDONE,
		xCLR_ALL => xCLR_ALL,
		xRESET_RAMP => xRESET_RAMP,
		xT_DATA	=> xT_DATA,
		xB_DATA	=> xB_DATA,
		xWADDR	=> WADDR,
		xTOUT 	=> xTOUT,
		xBOUT	 	=> xBOUT);	
--------------------------------------------------------------------------------
	xTARGET_RAM_TIMING : TARGET_RAM_TIMING 
	port map (
		xCLK 					=> xCLK_15MHz,--xCLK
		xDONE 				=> xDONE,
		xNRUN 				=> NRUN,
		xRAMP_DONE 			=> xRAMP_DONE,
		xCLR_ALL 			=> xCLR_ALL,
		xW_STRB	 			=> W_STRB,
		xSTART 				=> START,
		xRESET_RAMP 		=> xRESET_RAMP,
		xADDR 				=> WADDR);		
--------------------------------------------------------------------------------	
	xTARGET_RAM : TARGET_RAM 
	port map (
		xRADDR					=> xRADDR,
		xWADDR					=> WADDR,
		xSLWR 					=> xSLWR,
		xSTART 					=> START,
		xT_DATA					=> xT_DATA,
		xB_DATA					=> xB_DATA,
		xW_STRB 					=> W_STRB,
		xNRUN 					=> NRUN,
		xADC_T 					=> xADC_T,
		xADC_B					=> xADC_B);	
--------------------------------------------------------------------------------
	xBUF_START : BUF 
	port map (
		I  => START,
		O  => xSTART);	
--------------------------------------------------------------------------------
	xBUF_NRUN : BUF 
	port map (
		I  => NRUN,
		O  => xNRUN);	
--------------------------------------------------------------------------------
	xBUF_RAMP : BUF 
	port map (
		I  => RAMP,
		O  => xRAMP);
--------------------------------------------------------------------------------
	xBUF_TRIG : BUF 
	port map (
		I  => TRIG,
		O  => xTRIG);
--------------------------------------------------------------------------------
	xBUF_LOCK_ROW : BUF3
	port map (
		I	=> LOCK_ROW, 
		O	=> xLOCK_ROW);	
--------------------------------------------------------------------------------
	xBUF_LOCK_COL : BUF5
	port map (
		I	=> LOCK_COL, 
		O	=> xLOCK_COL);	
--------------------------------------------------------------------------------
end Behavioral;