--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--								University of Hawaii at Manoa						         --
--						Instrumentation Development Lab / GARY S. VARNER				--
--   								Watanabe Hall Room 214								      --
--  								  2505 Correa Road										   --
--  								 Honolulu, HI 96822											--
--  								Lab: (808) 956-2920											--
--	 								Fax: (808) 956-2930										   --
--  						E-mail: idlab@phys.hawaii.edu									   --
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------	
-- Design by: Larry L. Ruckman Jr.															--
-- DATE : 27 JUNE 2007																			--
-- Project name: ROBUSTv3 firmware															--
--	Module name: CH_TRIG		  																	--
--	Description : 																					--
-- 	Single Channel Trigger module															--
--		  											    												--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity TSA_32CYCLE_ONESHOT is
   port ( 
			 xEN					: in   std_logic;--SYNC to CLK
			 xCLK		 			: in   std_logic;
			 xTSA_STRB			: out  std_logic);
end TSA_32CYCLE_ONESHOT;

architecture Behavioral of TSA_32CYCLE_ONESHOT is
	type STATE_TYPE is ( PHASE0, PHASE1,PHASE2); 
	signal STATE			: STATE_TYPE := PHASE0;
	signal MAIN_CNT		: std_logic_vector(15 downto 0) := x"0000";
	signal TSA_STRB		: std_logic := '0';
--------------------------------------------------------------------------------
--   								components     		   						         --
	component BUF
    Port ( 
			  I   : in   std_logic;
			  O	: out  std_logic);
   end component;
--------------------------------------------------------------------------------	
begin
--------------------------------------------------------------------------------
	process(xCLK)
	begin
		if rising_edge(xCLK) then
--------------------------------------------------------------------------------			
			case STATE is
--------------------------------------------------------------------------------	
				when PHASE0 =>			
					MAIN_CNT <= (others=>'0');
					TSA_STRB <= '0';
					if xEN = '1' then
						TSA_STRB <= '1';
						STATE		<= PHASE1;
					end if;
--------------------------------------------------------------------------------	
				when PHASE1 =>			
					MAIN_CNT <= MAIN_CNT + 1;
					TSA_STRB <= '1';
					if MAIN_CNT = 17 then
						MAIN_CNT <= (others=>'0');
						TSA_STRB <= '0';
						STATE		<= PHASE2;
					end if;
--------------------------------------------------------------------------------	
				when PHASE2 =>			
					MAIN_CNT <= MAIN_CNT + 1;
					if MAIN_CNT = 15 then
						STATE		<= PHASE0;
					end if;
--------------------------------------------------------------------------------	
				when others =>	STATE<=PHASE0;																
			end case;
		end if;
	end process;					
--------------------------------------------------------------------------------
	xBUF_TSA_STRB : BUF 
	port map (
		I	=> TSA_STRB, 
		O	=> xTSA_STRB);		
--------------------------------------------------------------------------------
end Behavioral;